@@ -1407,6 +1407,7 @@ typedef enum
 	TYPE_FLEXIBLE_ARRAY,
 	TYPE_INFERRED_ARRAY,
 	TYPE_VECTOR,
+	TYPE_SIMD_VECTOR,
 	TYPE_INFERRED_VECTOR,
 	TYPE_LAST_ARRAYLIKE = TYPE_INFERRED_VECTOR,
 	TYPE_OPTIONAL,
@@ -1813,6 +1814,9 @@ static_assert(EXPR_LAST < 128, "Too many expression types");
 #define ALL_SIGNED_INTS TYPE_I8: case TYPE_I16: case TYPE_I32: case TYPE_I64: case TYPE_I128
 #define ALL_UNSIGNED_INTS TYPE_U8: case TYPE_U16: case TYPE_U32: case TYPE_U64: case TYPE_U128
 #define ALL_FLOATS TYPE_BF16: case TYPE_F16: case TYPE_F32: case TYPE_F64: case TYPE_F128
+#define ALL_VECTORS TYPE_VECTOR: case TYPE_INFERRED_VECTOR: case TYPE_SIMD_VECTOR
+#define ALL_ARRAYLIKE TYPE_VECTOR: case TYPE_INFERRED_VECTOR: case TYPE_SIMD_VECTOR: case TYPE_ARRAY: case TYPE_INFERRED_ARRAY: case TYPE_FLEXIBLE_ARRAY
+#define VECTORS TYPE_VECTOR: case TYPE_SIMD_VECTOR
 
 #define LOWERED_TYPES CT_TYPES: case TYPE_ENUM: case TYPE_ALIAS: case TYPE_TYPEID: \
   case TYPE_TYPEDEF: case TYPE_ANYFAULT: case TYPE_BITSTRUCT: \
