@@ -710,8 +710,8 @@ void expr_rewrite_to_const_zero(Expr *expr, Type *type)
 		case TYPE_STRUCT:
 		case TYPE_UNION:
 		case TYPE_BITSTRUCT:
-		case TYPE_VECTOR:
 		case TYPE_ARRAY:
+		case VECTORS:
 			expr_rewrite_const_initializer(expr, type, const_init_new_zero(type));
 			return;
 		case TYPE_TYPEDEF:
