@@ -36,7 +36,7 @@ static inline void begin_new_token(Lexer *lexer)
 #define peek_next(lexer_) ((lexer_)->current[1])
 
 // Is the current character '\0' if so we assume we reached the end.
-#define reached_end(lexer_) (lexer_->current[0] == '\0')
+#define reached_end(lexer_) ((lexer_)->current[0] == '\0')
 
 // Step one character forward and return that character
 INLINE char next(Lexer *lexer)
@@ -273,12 +273,19 @@ static void skip_whitespace(Lexer *lexer)
 				FALLTHROUGH;
 			case ' ':
 			case '\t':
-			case '\f':
 				next(lexer);
 				break;
 			case '\r':
 				// Already filtered out.
-				UNREACHABLE
+				UNREACHABLE_VOID
+			case '#':
+				if (lexer->file_begin == lexer->current && peek_next(lexer) == '!')
+				{
+					skip(lexer, 2);
+					parse_line_comment(lexer);
+					continue;
+				}
+				return;
 			default:
 				return;
 		}
@@ -351,6 +358,7 @@ EXIT:;
 		default:
 			break;
 	}
+	assert(type != TOKEN_INVALID_TOKEN);
 	return new_token(lexer, type, interned_string);
 }
 
@@ -375,6 +383,7 @@ static bool scan_number_suffix(Lexer *lexer, bool *is_float)
 	{
 		case 'l':
 			c = next(lexer);
+			if ((c | 32) == 'l') c = next(lexer);
 			if (*is_float)
 			{
 				return add_error_token_at_current(lexer, "Integer suffix '%c' is not valid for a floating point literal.", c);
@@ -389,6 +398,7 @@ static bool scan_number_suffix(Lexer *lexer, bool *is_float)
 			if ((c | 32) == 'l')
 			{
 				c = next(lexer);
+				if ((c | 32) == 'l') c = next(lexer);
 				break;
 			}
 			while (char_is_digit(c = peek(lexer))) next(lexer);
@@ -399,8 +409,17 @@ static bool scan_number_suffix(Lexer *lexer, bool *is_float)
 				return add_error_token_at_current(lexer, "Integer suffix '%c' is not valid for a floating point literal.", c);
 			}
 			next(lexer);
+			if (!char_is_digit(peek(lexer)))
+			{
+				return add_error_token_at_current(lexer, "Integer suffix 'i' must provide a bit width.");
+			}
+			next(lexer);
 			while (char_is_digit(c = peek(lexer))) next(lexer);
 			break;
+		case 'd':
+			c = next(lexer);
+			*is_float = true;
+			break;
 		case 'f':
 			next(lexer);
 			*is_float = true;
@@ -420,7 +439,7 @@ static bool scan_number_suffix(Lexer *lexer, bool *is_float)
 #define NEXT_AND_CHECK_NO_MULTIPLE_(lexer__) \
 	do { if (next(lexer__) == '_' && prev(lexer__) == '_') { \
 		return add_error_token_at_current(lexer__, "Multiple consecutive '_' are not allowed."); \
-	} } while(0);
+	} } while(0)
 /**
  * Parsing octals. Here we depart from the (error prone) C style octals with initial zero e.g. 0231
  * Instead we only support 0o prefix like 0o231. Note that lexing here doesn't actually parse the
@@ -806,7 +825,7 @@ static inline bool scan_char(Lexer *lexer)
 				// No escape, a regular character.
 				break;
 			default:
-				c = (signed char)escape;
+				c = (char)escape;
 				break;
 		}
 		// Default handling here:
@@ -1060,7 +1079,7 @@ static inline bool scan_hex_array(Lexer *lexer)
 	}
 	new_token(lexer, TOKEN_BYTES, lexer->lexing_start);
 	lexer->data.is_base64 = false;
-	lexer->data.bytes_len = (uint64_t)len / 2;
+	lexer->data.bytes_len = len / 2;
 	return true;
 }
 
@@ -1208,6 +1227,13 @@ EXIT:;
 	return true;
 }
 
+static bool next_is_ident(Lexer *lexer)
+{
+	size_t i = 0;
+	while (lexer->current[i] == '_') i++;
+	return char_is_lower(lexer->current[i]);
+}
+
 static bool lexer_scan_token_inner(Lexer *lexer)
 {
 	// Now skip the whitespace.
@@ -1238,7 +1264,8 @@ static bool lexer_scan_token_inner(Lexer *lexer)
 		case '"':
 			return scan_string(lexer);
 		case '#':
-			return scan_ident(lexer, TOKEN_HASH_IDENT, TOKEN_HASH_CONST_IDENT, TOKEN_HASH_TYPE_IDENT, '#');
+			if (!next_is_ident(lexer)) return new_token(lexer, TOKEN_HASH, "#");
+			return scan_ident(lexer, TOKEN_HASH_IDENT, TOKEN_INVALID_TOKEN, TOKEN_INVALID_TOKEN, '#');
 		case '$':
 			if (match(lexer, '$'))
 			{
@@ -1295,7 +1322,7 @@ static bool lexer_scan_token_inner(Lexer *lexer)
 		case '^':
 			return match(lexer, '=') ? new_token(lexer, TOKEN_BIT_XOR_ASSIGN, "^=") : new_token(lexer, TOKEN_BIT_XOR, "^");
 		case '?':
-			if (match(lexer, '?')) return new_token(lexer, TOKEN_QUESTQUEST, "??");
+			if (match(lexer, '?')) return match(lexer, '?') ? new_token(lexer, TOKEN_CT_TERNARY, "???") : new_token(lexer, TOKEN_QUESTQUEST, "??");
 			return match(lexer, ':') ? new_token(lexer, TOKEN_ELVIS, "?:") : new_token(lexer, TOKEN_QUESTION, "?");
 		case '<':
 			if (match(lexer, '<'))
@@ -1334,7 +1361,7 @@ static bool lexer_scan_token_inner(Lexer *lexer)
 		case '+':
 			if (match(lexer, '+'))
 			{
-				if (match(lexer, '+')) return new_token(lexer, TOKEN_CT_CONCAT, "+++");
+				if (match(lexer, '+')) return match(lexer, '=') ? new_token(lexer, TOKEN_CT_CONCAT_ASSIGN, "+++=") : new_token(lexer, TOKEN_CT_CONCAT, "+++");
 				return new_token(lexer, TOKEN_PLUSPLUS, "++");
 			}
 			if (match(lexer, '=')) return new_token(lexer, TOKEN_PLUS_ASSIGN, "+=");
