@@ -321,12 +321,12 @@ Type *type_infer_len_from_actual_type(Type *to_infer, Type *actual_type)
 			return type_add_optional(type_get_array(indexed, type_flatten(actual_type)->array.len), is_optional);
 		case TYPE_INFERRED_VECTOR:
 			if (!type_is_arraylike(type_flatten(actual_type))) return to_infer;
-			return type_add_optional(type_get_vector(indexed, type_flatten(actual_type)->array.len), is_optional);
+			return type_add_optional(type_get_vector(indexed, TYPE_VECTOR, type_flatten(actual_type)->array.len), is_optional);
 		case TYPE_SLICE:
 			return type_add_optional(type_get_slice(indexed), is_optional);
-		case TYPE_VECTOR:
+		case VECTORS:
 			// The case of int[*]*[<2>] x = ...
-			return type_add_optional(type_get_vector(indexed, to_infer->array.len), is_optional);
+			return type_add_optional(type_get_vector(indexed, to_infer->type_kind, to_infer->array.len), is_optional);
 		default:
 			UNREACHABLE
 	}
@@ -606,7 +606,7 @@ static void expr_recursively_rewrite_untyped_list(Expr *expr, Type *to_type)
 	}
 	switch (flat->type_kind)
 	{
-		case TYPE_VECTOR:
+		case VECTORS:
 		{
 			Type *indexed = type_get_indexed_type(to_type);
 			FOREACH(Expr *, e, values)
@@ -1105,7 +1105,7 @@ static bool rule_arr_to_vec(CastContext *cc, bool is_explicit, bool is_silent)
 		default:
 			return sema_cast_error(cc, false, is_silent);
 	}
-	cast_context_set_from(cc, type_get_vector(base, len));
+	cast_context_set_from(cc, type_get_vector(base, cc->to->type_kind, len));
 	return cast_is_allowed(cc, is_explicit, is_silent);
 }
 
@@ -1159,7 +1159,7 @@ static bool rule_slice_to_vecarr(CastContext *cc, bool is_explicit, bool is_sile
 		{
 			return report_cast_error(cc, false);
 		}
-		cast_context_set_from(cc, type_get_vector(cc->from->array.base, size));
+		cast_context_set_from(cc, type_get_vector(cc->from->array.base, cc->to->type_kind, size));
 	}
 	return cast_is_allowed(cc, is_explicit, is_silent);
 }
@@ -1393,6 +1393,23 @@ static bool rule_not_applicable(UNUSED CastContext *cc, UNUSED bool is_explicit,
 	UNREACHABLE
 }
 
+static bool rule_from_explicit_flattened(CastContext *cc, bool is_silent)
+{
+	Type *flat = type_flatten(cc->from);
+	if (cc->to_group == CONV_ANY || cc->to_group == CONV_INTERFACE)
+	{
+		switch (flat->type_kind)
+		{
+			case TYPE_ANY:
+			case TYPE_INTERFACE:
+				break;
+			default:
+				return sema_cast_error(cc, false, is_silent);
+		}
+	}
+	cast_context_set_from(cc, flat);
+	return cast_is_allowed(cc, true, is_silent);
+}
 
 static bool rule_from_distinct(CastContext *cc, bool is_explicit, bool is_silent)
 {
@@ -1402,8 +1419,7 @@ static bool rule_from_distinct(CastContext *cc, bool is_explicit, bool is_silent
 	// Explicit just flattens and tries again.
 	if (is_explicit)
 	{
-		cast_context_set_from(cc, type_flatten(from_type));
-		return cast_is_allowed(cc, is_explicit, is_silent);
+		return rule_from_explicit_flattened(cc, is_silent);
 	}
 	// No inline? Then it's an error.
 	if (!from_type->decl->is_substruct)
@@ -1601,9 +1617,7 @@ static bool rule_enum_to_value(CastContext *cc, bool is_explicit, bool is_silent
 		}
 		if (is_explicit)
 		{
-			cast_context_set_from(cc, type_flatten(inline_type));
-			// Explicit just flattens and tries again.
-			return cast_is_allowed(cc, is_explicit, is_silent);
+			return rule_from_explicit_flattened(cc, is_silent);
 		}
 		cast_context_set_from(cc, inline_type->canonical);
 		return cast_is_allowed(cc, is_explicit, is_silent);
@@ -1618,7 +1632,10 @@ static bool rule_enum_to_value(CastContext *cc, bool is_explicit, bool is_silent
 		}
 		// Use the inner type.
 		Type *inner = enum_decl->enums.type_info->type;
-		cast_context_set_from(cc, is_explicit ? type_flatten(inner) : inner);
+		if (is_explicit)
+		{
+			return rule_from_explicit_flattened(cc, is_silent);
+		}
 		return cast_is_allowed(cc, is_explicit, is_silent);
 	}
 
@@ -1695,6 +1712,7 @@ static void cast_ptr_to_any(Expr *expr, Type *type)
 {
 	Expr *inner = expr_copy(expr);
 	Expr *typeid = expr_copy(expr);
+	assert(type_no_optional(expr->type)->canonical->type_kind == TYPE_POINTER);
 	expr_rewrite_const_typeid(typeid, type_no_optional(expr->type)->canonical->pointer);
 	expr->expr_kind = EXPR_MAKE_ANY;
 	expr->make_any_expr = (ExprMakeAny) { .inner = inner, .typeid = typeid };
@@ -1766,7 +1784,7 @@ static void vector_const_initializer_convert_to_type(ConstInitializer *initializ
 			vector_const_initializer_convert_to_type(initializer->init_array_value.element, to_type);
 			break;
 	}
-	initializer->type = type_flatten(to_type);
+	const_init_set_type(initializer, to_type);
 }
 
 /**
@@ -1987,7 +2005,7 @@ static void cast_vec_to_arr(Expr *expr, Type *to_type)
 
 	ASSERT(expr->const_expr.const_kind == CONST_INITIALIZER);
 	ConstInitializer *list = expr->const_expr.initializer;
-	list->type = type_flatten(to_type);
+	const_init_set_type(list, to_type);
 	expr->type = to_type;
 }
 
@@ -2002,8 +2020,7 @@ static void cast_vec_to_vec(Expr *expr, Type *to_type)
 		// Extract indexed types.
 		Type *from_type = type_flatten(expr->type);
 		Type *from_element = from_type->array.base;
-		to_type = type_flatten(to_type);
-		Type *to_element = to_type->array.base;
+		Type *to_element = type_flatten(to_type)->array.base;
 
 		// float vec -> float/int/bool vec
 		if (type_is_float(from_element))
@@ -2391,7 +2408,7 @@ static void cast_arr_to_vec(Expr *expr, Type *to_type)
 {
 	Type *index_vec = type_flatten(type_get_indexed_type(to_type));
 	Type *index_arr = type_flatten(type_get_indexed_type(expr->type));
-	Type *to_temp = index_vec == index_arr ? to_type : type_get_vector(index_arr, type_flatten(expr->type)->array.len);
+	Type *to_temp = index_vec == index_arr ? to_type : type_get_vector(index_arr, to_type->canonical->type_kind, type_flatten(expr->type)->array.len);
 	if (sema_cast_const(expr))
 	{
 		// For the array -> vector this is always a simple rewrite of type.
@@ -2542,7 +2559,7 @@ CastRule cast_rules[CONV_LAST + 1][CONV_LAST + 1] = {
 
 CastFunction cast_function[CONV_LAST + 1][CONV_LAST + 1] = {
 //void,  wildcd, bool,    int, float,   ptr, slice,  vec,  bitst,  dist, array,struct,union,   any,  infc,  enum,  renum, func, typeid, anyfa,  vptr,  aptr, infer, ulist (to)
- {0,          0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0     }, // VOID (from)
+ {0,          0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0     }, // VOID (from)
  {XX2XX,      0, XX2XX, XX2XX, XX2XX, XX2XX, XX2XX, XX2XX, XX2XX,     0, XX2XX, XX2XX, XX2XX, XX2XX, XX2XX, XX2XX,     0, XX2XX, XX2XX, XX2XX, XX2XX, XX2XX,     0,     0     }, // WILDCARD
  {XX2VO,      0,     0, BO2IN, BO2FP,     0,     0, EX2VC,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,     0     }, // BOOL
  {XX2VO,      0, IN2BO, IN2IN, IN2FP, IN2PT,     0, EX2VC, IA2BS,     0,     0,     0,     0,     0,     0, IN2EN,     0, IN2PT,     0,     0, IN2PT, IN2PT,     0,     0     }, // INT
@@ -2611,13 +2628,14 @@ static ConvGroup group_from_type[TYPE_LAST + 1] = {
 	[TYPE_WILDCARD]         = CONV_WILDCARD,
 	[TYPE_TYPEINFO]         = CONV_NO,
 	[TYPE_MEMBER]           = CONV_NO,
+	[TYPE_SIMD_VECTOR]      = CONV_VECTOR,
 };
 
 INLINE ConvGroup type_to_group(Type *type)
 {
 	type = type->canonical;
 	if (type == type_voidptr) return CONV_VOIDPTR;
-	if (type->type_kind == TYPE_POINTER && (type->pointer->type_kind == TYPE_ARRAY || type->pointer->type_kind == TYPE_VECTOR)) return CONV_VAPTR;
+	if (type->type_kind == TYPE_POINTER && (type->pointer->type_kind == TYPE_ARRAY || type_kind_is_real_vector(type->pointer->canonical->type_kind))) return CONV_VAPTR;
 	if (type_len_is_inferred(type)) return CONV_INFERRED;
 	return group_from_type[type->type_kind];
 }
