@@ -658,15 +658,18 @@ static void sema_report_error_on_decl(SemaContext *context, NameResolve *name_re
 	}
 	if (!found && name_resolve->maybe_decl)
 	{
-		const char *maybe_name = decl_to_name(name_resolve->maybe_decl);
-		if (name_resolve->maybe_decl->unit->module->generic_module)
+		Decl *decl = name_resolve->maybe_decl;
+		Module *module = decl->unit->module;
+		const char *maybe_name = decl_to_name(decl);
+		Module *generic_module = module->generic_module;
+		if (!generic_module && module->is_generic) generic_module = module;
+		const char *module_name = generic_module ? generic_module->name->module : module->name->module;
+		if (generic_module && !name_resolve->is_parameterized)
 		{
-			const char *module_name = name_resolve->maybe_decl->unit->module->generic_module->name->module;
 			sema_error_at(context, span, "Did you mean the %s '%s' in the generic module %s? If so, use '%s{...}' instead.",
 			              maybe_name, symbol, module_name, symbol);
 			return;
 		}
-		const char *module_name = name_resolve->maybe_decl->unit->module->name->module;
 		if (path_name)
 		{
 			sema_error_at(context, span, "Did you mean the %s '%s::%s' in module %s? If so please add 'import %s'.",
@@ -965,14 +968,13 @@ bool sema_resolve_type_decl(SemaContext *context, Type *type)
 		case TYPE_BOOL:
 		case ALL_INTS:
 		case ALL_FLOATS:
+		case ALL_VECTORS:
 		case TYPE_ANYFAULT:
 		case TYPE_TYPEID:
 		case TYPE_POINTER:
 		case TYPE_FUNC_PTR:
 		case TYPE_UNTYPED_LIST:
 		case TYPE_MEMBER:
-		case TYPE_INFERRED_VECTOR:
-		case TYPE_VECTOR:
 		case TYPE_SLICE:
 		case TYPE_ANY:
 		case TYPE_INTERFACE:
@@ -1014,7 +1016,7 @@ Decl *sema_resolve_type_method(SemaContext *context, CanonicalType *type, const
 		{
 			case TYPE_ARRAY:
 				return declptrzero(methodtable_get(&compiler.context.method_extensions, type_get_inferred_array(type->array.base), method_name));
-			case TYPE_VECTOR:
+			case VECTORS:
 				return declptrzero(methodtable_get(&compiler.context.method_extensions, type_get_inferred_vector(type->array.base), method_name));
 			default:
 				return NULL;
@@ -1036,7 +1038,8 @@ Decl *sema_resolve_type_method(SemaContext *context, CanonicalType *type, const
 			type = type_decl->distinct->type->canonical;
 			goto RETRY;
 		case TYPE_ENUM:
-			type = type_decl->enums.type_info->type->canonical;
+		case TYPE_CONST_ENUM:
+			type = enum_inner_type(type);
 			goto RETRY;
 		default:
 			UNREACHABLE
