@@ -38,6 +38,7 @@ const char *attribute_list[NUMBER_OF_ATTRIBUTES];
 const char *builtin_list[NUMBER_OF_BUILTINS];
 const char *builtin_defines[NUMBER_OF_BUILTIN_DEFINES];
 const char *type_property_list[NUMBER_OF_TYPE_PROPERTIES];
+const char *kw_at_align;
 const char *kw_at_deprecated;
 const char *kw_at_ensure;
 const char *kw_at_enum_lookup;
@@ -46,6 +47,7 @@ const char *kw_at_param;
 const char *kw_at_pure;
 const char *kw_at_require;
 const char *kw_at_return;
+const char *kw_at_simd;
 const char *kw_in;
 const char *kw_inout;
 const char *kw_len;
@@ -280,6 +282,9 @@ void symtab_init(uint32_t capacity)
 	builtin_list[BUILTIN_STR_UPPER] = KW_DEF("str_upper");
 	builtin_list[BUILTIN_STR_LOWER] = KW_DEF("str_lower");
 	builtin_list[BUILTIN_STR_FIND] = KW_DEF("str_find");
+	builtin_list[BUILTIN_STR_PASCALCASE] = KW_DEF("str_pascalcase");
+	builtin_list[BUILTIN_STR_SNAKECASE] = KW_DEF("str_snakecase");
+	builtin_list[BUILTIN_STR_REPLACE] = KW_DEF("str_replace");
 	builtin_list[BUILTIN_SWIZZLE] = KW_DEF("swizzle");
 	builtin_list[BUILTIN_SWIZZLE2] = KW_DEF("swizzle2");
 	builtin_list[BUILTIN_SPRINTF] = KW_DEF("sprintf");
@@ -325,16 +330,19 @@ void symtab_init(uint32_t capacity)
 	kw_at_ensure = KW_DEF("@ensure");
 	kw_at_enum_lookup = KW_DEF("@enum_lookup_new");
 	kw_at_jump = KW_DEF("@jump");
+	kw_at_align = KW_DEF("@align");
+	kw_at_simd = KW_DEF("@simd");
 	kw_at_param = KW_DEF("@param");
 	kw_at_pure = KW_DEF("@pure");
 	kw_at_require = KW_DEF("@require");
 	kw_at_return = KW_DEF("@return");
-	attribute_list[ATTRIBUTE_ALIGN] = KW_DEF("@align");
+	attribute_list[ATTRIBUTE_ALIGN] = kw_at_align;
 	attribute_list[ATTRIBUTE_ALLOW_DEPRECATED] = KW_DEF("@allow_deprecated");
 	attribute_list[ATTRIBUTE_BENCHMARK] = KW_DEF("@benchmark");
 	attribute_list[ATTRIBUTE_BIGENDIAN] = KW_DEF("@bigendian");
 	attribute_list[ATTRIBUTE_BUILTIN] = KW_DEF("@builtin");
 	attribute_list[ATTRIBUTE_CALLCONV] = KW_DEF("@callconv");
+	attribute_list[ATTRIBUTE_CNAME] = KW_DEF("@cname");
 	attribute_list[ATTRIBUTE_COMPACT] = KW_DEF("@compact");
 	attribute_list[ATTRIBUTE_CONST] = KW_DEF("@const");
 	attribute_list[ATTRIBUTE_DEPRECATED] = KW_DEF("@deprecated");
@@ -375,6 +383,7 @@ void symtab_init(uint32_t capacity)
 	attribute_list[ATTRIBUTE_SAFEINFER] = KW_DEF("@safeinfer");
 	attribute_list[ATTRIBUTE_SAFEMACRO] = KW_DEF("@safemacro");
 	attribute_list[ATTRIBUTE_SECTION] = KW_DEF("@section");
+	attribute_list[ATTRIBUTE_SIMD] = kw_at_simd;
 	attribute_list[ATTRIBUTE_STRUCTLIKE] = KW_DEF("@structlike");
 	attribute_list[ATTRIBUTE_TEST] = KW_DEF("@test");
 	attribute_list[ATTRIBUTE_TAG] = KW_DEF("@tag");
